var cluster = require('cluster');

const client = require('prom-client');
const AggregatorRegistry = client.AggregatorRegistry;
const aggregatorRegistry = new AggregatorRegistry();
const masterRegistry = new client.Registry();

const defaultLabels = { 'application': 'fantasy_cluster_test_api_lobbyapi_matches_v1_getmatchplayerpoints' };
masterRegistry.setDefaultLabels(defaultLabels);
client.register.setDefaultLabels(defaultLabels);
// client.collectDefaultMetrics();

let clusterN = () => {
	const data = require('cluster');
	clusterN = () => data;
	return data;
};


const processCrashCounter = new client.Counter({
	name: 'process_crash',
	help: 'None',
	registers: [masterRegistry]
});

const externalMethodTimings = new client.Histogram({
    name: 'external',
    labelNames: ['status', 'method', 'url', 'worker_id'],
    help: 'None'
});

let port = 8081;
let master_port = 3000;
let counter = 0;
if (cluster.isMaster)
{
	let express = require('express');
	let app = express();
	let numCPUs = require("os").cpus().length;

	// Fork workers.
	for (let i = 0; i < numCPUs; i++)
	{
		cluster.fork();
	}
	cluster.on('fork', function (worker)
	{
		console.log('Worker ' + worker.process.pid + ' forked.');
	});
	cluster.on('exit', function (worker)
	{
		// processCrashCounter.inc();
		console.log('Worker DISCONNECTED ' + worker.process.pid + ' died. Replacing the died worker...');
		cluster.fork();
	});

	app.get('/master', async (req, res) => {
		try {
			const masterMetrics = masterRegistry.metrics();
			const workerMetrics = aggregatorRegistry.clusterMetrics();
			var start = new Date().getTime();
		
			Promise.all([workerMetrics, masterMetrics]).then(values => {
				var allMetrics = values[0]; 
				console.log("total time calculated from app: ", new Date().getTime() - start);
				res.set('Content-Type', aggregatorRegistry.contentType);
				res.send(allMetrics);
			}).catch(err => {
				console.log(err);
				res.send(err.message);
			});
		} catch (e) {
			res.statusCode = 500;
			res.send(e.message);
		}
	});

	app.listen(master_port, function(){
		console.log('master server listening on port %d', master_port);
	});
}
else
{
	let express = require('express');
	let app = express();



	app.use(function(req, res, next)
	{
		req.startTime = new Date().getTime();
		req.custom = req.url+"/"+counter%250
		counter++;
		next();
	});

	app.use(function(req, res, next){
		var startAt = req.startTime;
		res.on('finish', ()=> {
			try {
				var diff = new Date().getTime() - startAt;
				externalMethodTimings.labels({'method': req.method, 'url': req.custom}).observe(diff/1000.0);
				return;
			} catch(err){

			}
		});
		next();
	});

	app.get('/worker2', function(req, res){
		setTimeout(()=>{
			return res.status(200).send('worker Server is running');
		},20);
	});

	app.listen(port, function () {
		console.log('worker server listening on port %d', port);
	});
}